{% extends 'document_catalogue/base.html' %}
{% load mptt_tags %}

{% block dc-content %}

    {% include 'document_catalogue/include/categories.html' with categories=object_list show_documents=False %}

{% endblock %}
